@@ -1,6 +1,6 @@
 # -*- encoding: utf8 -*-
 #
-# $Id: html.py 4869 2010-10-07 03:48:53Z rjones $
+# $Id: html.py 5358 2011-05-12 03:10:48Z rjones $
 # $HeadURL: svn+ssh://svn/svn/trunk/api/eklib/html.py $
 #
 '''Simple, elegant HTML generation.
@@ -52,26 +52,22 @@ between sub-tags to generate a nicer output. You can also see in that
 example the chaining of tags in ``l.li.b``. If you wished you could add
 attributes to those chained tags, eg: ``l.li(id="special").b``.
 
-The alternative to the above method is to use the containter tag as a
-context for adding the sub-tags. The top-level ``HTML`` object keeps track
-of which tag is the current context:
+Tag attributes may be passed in as well:
 
->>> with h.table(border='1'):
-...   for i in range(2):
-...     with h.tr:
-...       h.td('column 1')
-...       h.td('column 2')
-...  print h
+>>> t = h.table(border='1')
+>>> for i in range(2):
+>>>   r = t.tr
+>>>   r.td('column 1')
+>>>   r.td('column 2')
+>>> print t
 <table border="1">
 <tr><td>column 1</td><td>column 2</td></tr>
 <tr><td>column 1</td><td>column 2</td></tr>
 </table>
 
-Note the addition of an attribute to the ``<table>`` tag.
-
-A variation on the above is to explicitly reference the context variable,
-but then there's really no benefit to using a ``with`` statement. The
-following is functionally identical to the first list construction:
+A variation on the above is to use a tag as a context variable. The
+following is functionally identical to the first list construction but
+with a slightly different sytax emphasising the HTML structure:
 
 >>> with h.ol as l:
 ...   l.li('item 1')
@@ -155,9 +151,26 @@ with the appropriate XHTML minimized tag syntax. For example:
 <p></p>
 <br />
 
+
+Creating XML
+------------
+
+A slight tweak to the ``html.XHTML()`` implementation allows us to generate
+arbitrary XML using ``html.XML()``:
+
+>>> from html import XML
+>>> h = XML()
+>>> h.p
+>>> h.br('hi there')
+>>> print h
+<p />
+<br>hi there</br>
+
+
 Version History (in Brief)
 --------------------------
 
+- 1.14 added plain XML support
 - 1.13 allow adding (X)HTML instances (tags) as new document content
 - 1.12 fix handling of XHTML empty tags when generating unicode
   output (thanks Carsten Eggers)
@@ -176,12 +189,15 @@ Version History (in Brief)
 
 ----
 
+I would be interested to know whether this module is useful - if you use it
+please indicate so at https://www.ohloh.net/p/pyhtml
+
 This code is copyright 2009-2010 eKit.com Inc (http://www.ekit.com/)
 See the end of the source file for the license of use.
 XHTML support was contributed by Michael Haubenwallner.
 '''
 from __future__ import with_statement
-__version__ = '1.13'
+__version__ = '1.14'
 
 import unittest
 import cgi
@@ -212,7 +228,7 @@ class HTML(object):
         if name == 'newline':
             e = '\n'
         else:
-            e = HTML(name, self.stack)
+            e = self.__class__(name, self.stack)
         if self.top:
             self.stack[-1].content.append(e)
         else:
@@ -292,24 +308,10 @@ class HTML(object):
 
 class XHTML(HTML):
     '''Easily generate XHTML.
-
     '''
-
     empty_elements = set('base meta link hr br param img area input col \
         colgroup basefont isindex frame'.split())
 
-    def __getattr__(self, name):
-        # adding a new tag or newline
-        if name == 'newline':
-            e = '\n'
-        else:
-            e = XHTML(name, self.stack)
-        if self.top:
-            self.stack[-1].content.append(e)
-        else:
-            self.content.append(e)
-        return e
-
     def stringify(self, str_type):
         # turn me and my content into text
         # honor empty and non-empty elements
@@ -327,6 +329,30 @@ class XHTML(HTML):
         return s
 
 
+class XML(XHTML):
+    '''Easily generate XML.
+
+    All tags with no contents are reduced to self-terminating tags.
+    '''
+    newline_default_on = set()          # no tags are special
+
+    def stringify(self, str_type):
+        # turn me and my content into text
+        # honor empty and non-empty elements
+        join = '\n' if self.newlines else ''
+        if self.name is None:
+            return join.join(map(str_type, self.content))
+        a = ['%s="%s"' % i for i in self.attrs.items()]
+        l = [self.name] + a
+        s = '<%s>%s' % (' '.join(l), join)
+        if self.content:
+            s += join.join(map(str_type, self.content))
+            s += join + '</%s>' % self.name
+        else:
+            s = '<%s />%s' % (' '.join(l), join)
+        return s
+
+
 class TestCase(unittest.TestCase):
     def test_empty_tag(self):
         'generation of an empty HTML tag'
@@ -360,6 +386,12 @@ class TestCase(unittest.TestCase):
         'generate XHTML for just one tag'
         self.assertEquals(str(XHTML().br), '<br />')
 
+    def test_xml(self):
+        'generate XML'
+        self.assertEquals(str(XML().br), '<br />')
+        self.assertEquals(str(XML().p), '<p />')
+        self.assertEquals(str(XML().br('text')), '<br>text</br>')
+
     def test_para_tag(self):
         'generation of a tag with contents'
         h = HTML()
